@@ -343,6 +343,8 @@ class geom2graph:
             return
             
         model, results = self.fill_bonds()
+        if not model:
+            return
         
         self.solver_results = {'model': model, 'results': results}
         
@@ -351,35 +353,35 @@ class geom2graph:
             for index in model.edge_weights_ads:
                 self.graph.edges[index]['weight'] = int(model.edge_weights_ads[index].value)
                 
-            
-            edges_pt_dict = {i: model.edge_weights_pts[i].value for i in model.edge_weights_pts}          
-            
-            nodes = set([i for sub in model.edges_pts for i in sub])
-            not_pt_nodes = [i for i in nodes if self.symbols[i] != self.surface_type]
-            
-            edge_pt_ads_sort = dict.fromkeys(not_pt_nodes, {})
-            
-            for index in model.edge_weights_pts:
-                intersection = set(index).intersection(not_pt_nodes)
-                if intersection:
-                    edge_pt_ads_sort[list(intersection)[0]][index] = model.edge_weights_pts[index].value
+            if self.surface_type in [self.graph.nodes[i]['element'] for i in self.graph.nodes]:
+                
+                edges_pt_dict = {i: model.edge_weights_pts[i].value for i in model.edge_weights_pts}          
+                nodes = set([i for sub in model.edges_pts for i in sub])
+                not_pt_nodes = [i for i in nodes if self.symbols[i] != self.surface_type]
+                
+                edge_pt_ads_sort = dict.fromkeys(not_pt_nodes, {})
+                
+                for index in model.edge_weights_pts:
+                    intersection = set(index).intersection(not_pt_nodes)
+                    if intersection:
+                        edge_pt_ads_sort[list(intersection)[0]][index] = model.edge_weights_pts[index].value
+                        
+                for node in edge_pt_ads_sort:
                     
-            for node in edge_pt_ads_sort:
+                    edge_sum = 0
+                    edges= 0
+                    
+                    for edge in edge_pt_ads_sort[node]:
+                        edge_sum += edge_pt_ads_sort[node][edge]
+                        edges += 1
+                    edge_sum = edge_sum/edges
+                    
+                    for edge in edge_pt_ads_sort[node]:
+                        edges_pt_dict[edge] = edge_sum
                 
-                edge_sum = 0
-                edges= 0
+                for index in edges_pt_dict: 
+                    self.graph.edges[index]['weight'] = round(edges_pt_dict[index],3)
                 
-                for edge in edge_pt_ads_sort[node]:
-                    edge_sum += edge_pt_ads_sort[node][edge]
-                    edges += 1
-                edge_sum = edge_sum/edges
-                
-                for edge in edge_pt_ads_sort[node]:
-                    edges_pt_dict[edge] = edge_sum
-            
-            for index in edges_pt_dict: 
-                self.graph.edges[index]['weight'] = round(edges_pt_dict[index],3)
-            
             
         else:
             # for index in self.graph.edges:
@@ -416,6 +418,8 @@ class geom2graph:
             else:
                 nodes_ads[node] = node_dict[node]['valency']
 
+        if not edges_ads and not edges_pts:
+            return None, None
         
         model = pe.ConcreteModel()
         
@@ -447,23 +451,28 @@ class geom2graph:
                                             bounds = (0.01, 4))
         
             def valency_cap(model, i):
-            
-                node_in_edge_ads = sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j])
-                node_in_edge_hs = sum([model.edge_weights_hs[j] for j in model.edge_weights_hs if i in j])
-                node_in_edge_pts = sum([model.edge_weights_pts[j] for j in model.edge_weights_pts if i in j])
                 
-                return model.valency[i] == node_in_edge_ads + node_in_edge_hs + node_in_edge_pts
+                edge_sum = 0
+                
+                if edges_ads:
+                    edge_sum += sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j]) 
+                if edges_hs:
+                    edge_sum += sum([model.edge_weights_hs[j] for j in model.edge_weights_hs if i in j])
+                if edges_pts:
+                    edge_sum += sum([model.edge_weights_pts[j] for j in model.edge_weights_pts if i in j])
+                
+                return model.valency[i] == edge_sum
                        
         
             def edge_sum(model):
                 
                 total_ads = sum([(model.edge_weights_ads[i]) for i in model.edges_ads])
                 total_pts = sum([(model.edge_weights_pts[i]) for i in model.edges_pts])
+                
                 total = total_ads + total_pts
+                
                 return total
             
-
-            
         else:
             
             def valency_cap(model, i):
@@ -481,65 +490,35 @@ class geom2graph:
 
 
         model.obj = pe.Objective(sense = pe.maximize, rule = edge_sum)
-        model.con = pe.Constraint(model.nodes_ads, rule = valency_cap)
-            
+
+
+        constraint_node_ads = set([i for sub in model.edges_ads for i in sub])
+        
+        if edges_pts:
+            constraint_nodes_pts = set([i for sub in model.edges_pts for i in sub])
+            constraint_nodes = [i for i in constraint_nodes.union(constraint_nodes_pts) if self.symbols[i] != self.surface_type]
+        
+        model.con = pe.Constraint(constraint_nodes, rule = valency_cap)
+
         solver = po.SolverFactory('glpk')
         
-        # try:
-        print('Solving ' + str(self.atoms[self.adsorbate_atoms].symbols))
-        results = solver.solve(model)
-            
-        # except:
-        #     print('Solver unable to get solution')
-        #     model = None
-        #     results = None
+        try:
+            print('Solving ' + str(self.atoms[self.adsorbate_atoms].symbols))
+            results = solver.solve(model)
+                
+        except:
+            print('Solver unable to get solution')
+            model = None
+            results = None
         
         return model, results
 
-    def draw_graph(self):
+    def draw_graph(self, cutoff = 0.3, 
+                        node_label_type = 'element', edge_labels = False, edge_label_type = 'weight'):
         
-        
-        """
-        Displays the graph
-        :param graph: networkx graph object
-        :return: None
-        """
-        
-        nodes = self.graph.nodes()
-        
-        xs = np.linspace(-0.7, 0.7, len(nodes))
-        
-        positions = {i: (xs[count], np.random.uniform(low = -0.5, high = 1)) for count, i in enumerate(nodes)}
-        
-        Pt = [i for i in nodes if self.symbols[i] == self.surface_type]
-        
-        if len(Pt) ==1:
-            spaced = (0,-1)
-            positions[Pt[0]] = spaced
-            fixed_positions = {Pt[0]: tuple(spaced)}
-            pos = nx.spring_layout(self.graph, pos = fixed_positions, fixed = fixed_positions, weight=None)
-            
-        elif len(Pt)>1:
-            spaced = np.vstack((np.linspace(-0.95, 0.95, len(Pt)), np.ones(len(Pt))*-1)).T
-            for count, i in enumerate(Pt):
-                positions[i] = tuple(spaced[count])
-            fixed_positions = {i: tuple(spaced[count]) for count, i in enumerate(Pt)}
-        
-            pos = nx.spring_layout(self.graph, pos = positions, fixed = fixed_positions, weight=None)
-            
-        
-        else:
-            pos = nx.spring_layout(self.graph, pos = positions, weight=None)
-        
-        nx.draw(self.graph, with_labels=False, node_size=1000, node_color="skyblue",
-                node_shape="o", alpha=0.5, linewidths=4, font_size=15,
-                font_color="black", font_weight="bold", width=2, edge_color="grey",
-                pos=pos)
-        node_labels = nx.get_node_attributes(self.graph, 'element')
-        edge_labels = nx.get_edge_attributes(self.graph, 'weight')
-        nx.draw_networkx_labels(self.graph, pos, node_labels)
-        nx.draw_networkx_edge_labels(self.graph, pos, edge_labels)
-        plt.show()
+        from utils import draw_surf_graph
+        draw_surf_graph(self.graph, cutoff = cutoff, 
+                            node_label_type = node_label_type, edge_labels = edge_labels, edge_label_type = edge_label_type)
         
     def get_SMILES_string(self):    
         self.smiles_string = pysmiles.write_smiles(self.graph)
@@ -555,9 +534,10 @@ class geom2graph:
                 self.graph.nodes[n[0]]['element']='M'
         
 if __name__=="__main__":
-    # vasp = geom2graph(vasp = 'vasp_to_network_tests/CONTCARmethane')
-    #vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCARpraneetalphahdown')
-    vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCAR_CHCH_holhol')
+    # vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCARmethane')
+    # vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCARpraneetalphahdown')
+    # vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCAR_CHCH_holhol')
+    vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCAR_Ir_CH3_hol', surface_type = 'Ir')
     
     # vasp.find_active_site_x_nearest_neighbors(2)
     
@@ -572,4 +552,4 @@ if __name__=="__main__":
     vasp.get_gcn()
     print(vasp.gcn)
     
-    vasp.draw_graph()
+    vasp.draw_graph(edge_labels=True)
